@@ -700,7 +700,7 @@ switch (op) {                                           /* case on opcode */
         fpd = 1;                                        /* set FPD */
         R[4] = R[4] & 0377;                             /* mask fill */
         c = t = 0;
-        for (i = 0; (R[0] || R[2]); ) {                   /* until cnts == 0 */
+        for (i = 0; (R[0] || R[2]); ) {                 /* until cnts == 0 */
             if (R[0]) c = ReadB (R[1] | dsenable);      /* get src1 or fill */
             else c = R[4];
             if (R[2]) t = ReadB (R[3] | dsenable);      /* get src2 or fill */
@@ -1192,15 +1192,18 @@ return TestDstr (src);                                  /* clean -0 */
         dsrc    =       decimal string descriptor
         src     =       decimal string structure
         flag    =       numeric/packed flag
+
    PSW.NZ are also set to their proper values
    PSW.V will be set on overflow; it must be initialized elsewhere
-        (to allow for external overflow calculations)
+   (to allow for external overflow calculations)
 
    The rules for the stored sign and the PSW sign are:
-        - Stored sign is negative if input is negative, string type
-          is signed, and the result is non-zero or there was overflow
-        - PSW sign is negative if input is negative, string type is
-          signed, and the result is non-zero
+
+   - Stored sign is negative if input is negative, string type
+     is signed, and the result is non-zero or there was overflow
+   - PSW sign is negative if input is negative, string type is
+     signed, and the result is non-zero
+
    Thus, the stored sign and the PSW sign will differ in one case:
    a negative zero generated by overflow is stored with a negative
    sign, but PSW.N is clear
@@ -1265,16 +1268,18 @@ return;
         cy      =       carry in
    Output       =       1 if carry, 0 if no carry
 
-   This algorithm courtesy Anton Chernoff, circa 1992 or even earlier
+   This algorithm courtesy Anton Chernoff, circa 1992 or even earlier.
 
    We trace the history of a pair of adjacent digits to see how the
    carry is fixed; each parenthesized item is a 4b digit.
 
    Assume we are adding:
+
         (a)(b)  I
    +    (x)(y)  J
 
    First compute I^J:
+
         (a^x)(b^y)      TMP
 
    Note that the low bit of each digit is the same as the low bit of
@@ -1283,6 +1288,7 @@ return;
 
    Now compute I+J+66 to get decimal addition with carry forced left
    one digit:
+
         (a+x+6+carry mod 16)(b+y+6 mod 16)      SUM
 
    Note that if there was a carry from b+y+6, then the low bit of the
@@ -1357,6 +1363,7 @@ return 0;
 
    Arguments:
         dsrc    =       decimal string structure
+
    Returns the non-zero length of the string, in int32 units
    If the string is zero, the sign is cleared
 */
@@ -1395,8 +1402,8 @@ return ((nz - 1) * 8) + i;
         mtable[10] =    array of decimal string structures
 
    Note that dsrc has a high order zero nibble; this
-        guarantees that the largest multiple won't overflow
-   Also note that mtable[0] is not filled in
+   guarantees that the largest multiple won't overflow.
+   Also note that mtable[0] is not filled in.
 */
 
 void CreateTable (DSTR *dsrc, DSTR mtable[10])
@@ -1560,7 +1567,7 @@ fpd = 0;                                                /* instr done */
 return;
 }
     
-/* Test for CIS mid-instruction interrupt - stub for now */
+/* Test for CIS mid-instruction interrupt */
 
 t_bool cis_int_test (int32 cycles, int32 oldpc, t_stat *st)
 {
