@@ -1,6 +1,6 @@
 /* pdp11_dz.c: DZ11 terminal multiplexor simulator
 
-   Copyright (c) 2001-2005, Robert M Supnik
+   Copyright (c) 2001-2006, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,6 +25,8 @@
 
    dz           DZ11 terminal multiplexor
 
+   29-Oct-06    RMS     Synced poll and clock
+   22-Nov-05    RMS     Revised for new terminal processing routines
    07-Jul-05    RMS     Removed extraneous externs
    15-Jun-05    RMS     Revised for new autoconfigure interface
    04-Apr-04    RMS     Added per-line logging
@@ -61,12 +63,12 @@ extern int32 int_req;
 
 #elif defined (VM_VAX)                                  /* VAX version */
 #include "vax_defs.h"
-#define DZ_8B_DFLT      UNIT_8B
+#define DZ_8B_DFLT      TT_MODE_8B
 extern int32 int_req[IPL_HLVL];
 
 #else                                                   /* PDP-11 version */
 #include "pdp11_defs.h"
-#define DZ_8B_DFLT      UNIT_8B
+#define DZ_8B_DFLT      TT_MODE_8B
 extern int32 int_req[IPL_HLVL];
 #endif
 
@@ -80,9 +82,6 @@ extern int32 int_req[IPL_HLVL];
 #define DZ_LINES        8
 #endif
 
-#define UNIT_V_8B       (UNIT_V_UF + 0)                 /* 8b output */
-#define UNIT_8B         (1 << UNIT_V_8B)
-
 #define DZ_MNOMASK      (DZ_MUXES - 1)                  /* mask for mux no */
 #define DZ_LNOMASK      (DZ_LINES - 1)                  /* mask for lineno */
 #define DZ_LMASK        ((1 << DZ_LINES) - 1)           /* mask of lines */
@@ -217,8 +216,9 @@ REG dz_reg[] = {
     };
 
 MTAB dz_mod[] = {
-    { UNIT_8B, 0, "7b", "7B", NULL },
-    { UNIT_8B, UNIT_8B, "8b", "8B", NULL },
+    { TT_MODE, TT_MODE_7B, "7b", "7B", NULL },
+    { TT_MODE, TT_MODE_8B, "8b", "8B", NULL },
+    { TT_MODE, TT_MODE_7P, "7p", "7P", NULL },
     { MTAB_XTD | MTAB_VDV, 1, NULL, "DISCONNECT",
       &tmxr_dscln, NULL, &dz_desc },
     { UNIT_ATT, UNIT_ATT, "connections", NULL, NULL, &dz_summ },
@@ -295,7 +295,7 @@ return SCPE_OK;
 t_stat dz_wr (int32 data, int32 PA, int32 access)
 {
 int32 dz = ((PA - dz_dib.ba) >> 3) & DZ_MNOMASK;        /* get mux num */
-int32 i, line;
+int32 i, c, line;
 TMLN *lp;
 
 switch ((PA >> 1) & 03) {                               /* case on PA<2:1> */
@@ -305,7 +305,8 @@ switch ((PA >> 1) & 03) {                               /* case on PA<2:1> */
             (dz_csr[dz] & 0377) | (data << 8):
             (dz_csr[dz] & ~0377) | data;
         if (data & CSR_CLR) dz_clear (dz, FALSE);       /* clr? reset */
-        if (data & CSR_MSE) sim_activate (&dz_unit, tmxr_poll);
+        if (data & CSR_MSE)                             /* MSE? start poll */
+            sim_activate (&dz_unit, clk_cosched (tmxr_poll));
         else dz_csr[dz] &= ~(CSR_SA | CSR_RDONE | CSR_TRDY);
         if ((data & CSR_RIE) == 0) dz_clr_rxint (dz);   /* RIE = 0? */
         else if (((dz_csr[dz] & CSR_IE) == 0) &&        /* RIE 0->1? */
@@ -364,8 +365,8 @@ switch ((PA >> 1) & 03) {                               /* case on PA<2:1> */
         if (dz_csr[dz] & CSR_MSE) {                     /* enabled? */
             line = (dz * DZ_LINES) + CSR_GETTL (dz_csr[dz]);
             lp = &dz_ldsc[line];                        /* get line desc */
-            tmxr_putc_ln (lp, dz_tdr[dz] &              /* store char */
-                ((dz_unit.flags & UNIT_8B)? 0377: 0177));
+            c = sim_tt_outcvt (dz_tdr[dz], TT_GET_MODE (dz_unit.flags));
+            if (c >= 0) tmxr_putc_ln (lp, c);           /* store char */
             tmxr_poll_tx (&dz_desc);                    /* poll output */
             dz_update_xmti ();                          /* update int */
             }
