@@ -1,6 +1,6 @@
 /* pdp18b_stddev.c: 18b PDP's standard devices
 
-   Copyright (c) 1993-2005, Robert M Supnik
+   Copyright (c) 1993-2006, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,16 @@
    tto          teleprinter
    clk          clock
 
+   18-Oct-06    RMS     Added PDP-15 programmable duplex control
+                        Fixed handling of non-printable characters in KSR mode
+                        Changed clock to be free-running
+                        Fixed out-of-tape behavior for PDP-9 vs PDP-15
+                        Synced keyboard to clock
+   30-Jun-06    RMS     Fixed KSR-28 shift tracking
+   20-Jun-06    RMS     Added KSR ASCII reader support
+   13-Jun-06    RMS     Fixed Baudot letters/figures inversion for PDP-4
+                        Fixed PDP-4/PDP-7 default terminal to be local echo
+   22-Nov-05    RMS     Revised for new terminal processing routines
    28-May-04    RMS     Removed SET TTI CTRL-C
    16-Feb-04    RMS     Fixed bug in hardware read-in mode bootstrap
    14-Jan-04    RMS     Revised IO device call interface
@@ -65,6 +75,8 @@
 
 #define UNIT_V_RASCII   (UNIT_V_UF + 0)                 /* reader ASCII */
 #define UNIT_RASCII     (1 << UNIT_V_RASCII)
+#define UNIT_V_KASCII   (UNIT_V_UF + 1)                 /* KSR ASCII */
+#define UNIT_KASCII     (1 << UNIT_V_KASCII)
 #define UNIT_V_PASCII   (UNIT_V_UF + 0)                 /* punch ASCII */
 #define UNIT_PASCII     (1 << UNIT_V_PASCII)
 
@@ -77,10 +89,43 @@ extern UNIT cpu_unit;
 int32 clk_state = 0;
 int32 ptr_err = 0, ptr_stopioe = 0, ptr_state = 0;
 int32 ptp_err = 0, ptp_stopioe = 0;
-int32 tti_state = 0;
-int32 tto_state = 0;
+int32 tti_2nd = 0;                                      /* 2nd char waiting */
+int32 tty_shift = 0;                                    /* KSR28 shift state */
+int32 tti_fdpx = 0;                                     /* prog mode full duplex */
 int32 clk_tps = 60;                                     /* ticks/second */
 int32 tmxr_poll = 16000;                                /* term mux poll */
+uint32 clk_task_last = 0;
+uint32 clk_task_timer = 0;
+
+const int32 asc_to_baud[128] = {
+    000,000,000,000,000,000,000,064,                    /* bell */
+    000,000,0110,000,000,0102,000,000,                  /* lf, cr */
+    000,000,000,000,000,000,000,000,
+    000,000,000,000,000,000,000,000,
+    0104,066,061,045,062,000,053,072,                   /* space - ' */
+    076,051,000,000,046,070,047,067,                    /* ( - / */
+    055,075,071,060,052,041,065,074,                    /* 0 - 7 */
+    054,043,056,057,000,000,000,063,                    /* 8 - ? */
+    000,030,023,016,022,020,026,013,                    /* @ - G */
+    005,014,032,036,011,007,006,003,                    /* H - O */
+    015,035,012,024,001,034,017,031,                    /* P - W */
+    027,025,021,000,000,000,000,000,                    /* X - _ */
+    000,030,023,016,022,020,026,013,                    /* ` - g */
+    005,014,032,036,011,007,006,003,                    /* h - o */
+    015,035,012,024,001,034,017,031,                    /* p - w */
+    027,025,021,000,000,000,000,000                     /* x - DEL */
+    };
+
+const char baud_to_asc[64] = {
+     0 ,'T',015,'O',' ','H','N','M',
+    012,'L','R','G','I','P','C','V',
+    'E','Z','D','B','S','Y','F','X',
+    'A','W','J', 0 ,'U','Q','K', 0,
+     0 ,'5','\r','9',' ','#',',','.',
+    012,')','4','&','8','0',':',';',
+    '3','"','$','?','\a','6','!','/',
+    '-','2','\'',0 ,'7','1','(', 0
+    };
 
 int32 ptr (int32 dev, int32 pulse, int32 dat);
 int32 ptp (int32 dev, int32 pulse, int32 dat);
@@ -109,6 +154,7 @@ t_stat ptr_boot (int32 unitno, DEVICE *dptr);
 t_stat tty_set_mode (UNIT *uptr, int32 val, char *cptr, void *desc);
 t_stat clk_set_freq (UNIT *uptr, int32 val, char *cptr, void *desc);
 t_stat clk_show_freq (FILE *st, UNIT *uptr, int32 val, void *desc);
+int32 clk_task_upd (t_bool clr);
 
 extern int32 upd_iors (void);
 
@@ -127,6 +173,10 @@ REG clk_reg[] = {
     { FLDATA (INT, int_hwre[API_CLK], INT_V_CLK) },
     { FLDATA (DONE, int_hwre[API_CLK], INT_V_CLK) },
     { FLDATA (ENABLE, clk_state, 0) },
+#if defined (PDP15)
+    { ORDATA (TASKTIMER, clk_task_timer, 18) },
+    { DRDATA (TASKLAST, clk_task_last, 32), REG_HRO },
+#endif
     { DRDATA (TIME, clk_unit.wait, 24), REG_NZ + PV_LEFT },
     { DRDATA (TPS, clk_tps, 8), PV_LEFT + REG_HRO },
     { NULL }
@@ -180,6 +230,8 @@ REG ptr_reg[] = {
     };
 
 MTAB ptr_mod[] = {
+    { UNIT_RASCII, UNIT_RASCII, "even parity ASCII", NULL },
+    { UNIT_KASCII, UNIT_KASCII, "forced parity ASCII", NULL },
     { MTAB_XTD|MTAB_VDV, 0, "DEVNO", NULL, NULL, &show_devno },
     { 0 }
     };
@@ -219,6 +271,7 @@ REG ptp_reg[] = {
     };
 
 MTAB ptp_mod[] = {
+    { UNIT_PASCII, UNIT_PASCII, "7b ASCII", NULL },
     { MTAB_XTD|MTAB_VDV, 0, "DEVNO", NULL, NULL, &show_devno },
     { 0 }
     };
@@ -236,76 +289,52 @@ DEVICE ptp_dev = {
    tti_dev      TTI device descriptor
    tti_unit     TTI unit
    tti_reg      TTI register list
-   tti_trans    ASCII to Baudot table
 */
 
 #if defined (KSR28)
 #define TTI_WIDTH       5
 #define TTI_FIGURES     (1 << TTI_WIDTH)
-#define TTI_2ND         (1 << (TTI_WIDTH + 1))
-#define TTI_BOTH        (1 << (TTI_WIDTH + 2))
-#define BAUDOT_LETTERS  033
-#define BAUDOT_FIGURES  037
+#define TTI_BOTH        (1 << (TTI_WIDTH + 1))
+#define BAUDOT_LETTERS  037
+#define BAUDOT_FIGURES  033
 
-static const int32 tti_trans[128] = {
-    000,000,000,000,000,000,000,064,                    /* bell */
-    000,000,0210,000,000,0202,000,000,                  /* lf, cr */
-    000,000,000,000,000,000,000,000,
-    000,000,000,000,000,000,000,000,
-    0204,066,061,045,062,000,053,072,                   /* space - ' */
-    076,051,000,000,046,070,047,067,                    /* ( - / */
-    055,075,071,060,052,041,065,074,                    /* 0 - 7 */
-    054,043,056,057,000,000,000,063,                    /* 8 - ? */
-    000,030,023,016,022,020,026,013,                    /* @ - G */
-    005,014,032,036,011,007,006,003,                    /* H - O */
-    015,035,012,024,001,034,017,031,                    /* P - W */
-    027,025,021,000,000,000,000,000,                    /* X - _ */
-    000,030,023,016,022,020,026,013,                    /* ` - g */
-    005,014,032,036,011,007,006,003,                    /* h - o */
-    015,035,012,024,001,034,017,031,                    /* p - w */
-    027,025,021,000,000,000,000,000                     /* x - DEL */
-    };
 #else
 
 #define TTI_WIDTH       8
 #endif
 
 #define TTI_MASK        ((1 << TTI_WIDTH) - 1)
-#define UNIT_V_8B       (UNIT_V_UF + 0)                 /* 8B */
-#define UNIT_V_KSR      (UNIT_V_UF + 1)                 /* KSR33 */
-#define UNIT_V_HDX      (UNIT_V_UF + 2)                 /* half duplex */
-#define UNIT_8B         (1 << UNIT_V_8B)
-#define UNIT_KSR        (1 << UNIT_V_KSR)
-#define UNIT_HDX        (1 << UNIT_V_HDX)
+#define TTUF_V_HDX      (TTUF_V_UF + 0)                 /* half duplex */
+#define TTUF_HDX        (1 << TTUF_V_HDX)
 
 DIB tti_dib = { DEV_TTI, 1, &tti_iors, { &tti } };
 
-#if defined (PDP4) || defined (PDP7)
-UNIT tti_unit = { UDATA (&tti_svc, UNIT_KSR, 0), KBD_POLL_WAIT };
-#else
-UNIT tti_unit = { UDATA (&tti_svc, UNIT_KSR+UNIT_HDX, 0), KBD_POLL_WAIT };
-#endif
+UNIT tti_unit = { UDATA (&tti_svc, TT_MODE_KSR+TTUF_HDX, 0), 0 };
 
 REG tti_reg[] = {
     { ORDATA (BUF, tti_unit.buf, TTI_WIDTH) },
+#if defined (KSR28)
+    { ORDATA (BUF2ND, tti_2nd, TTI_WIDTH), REG_HRO },
+#endif
     { FLDATA (INT, int_hwre[API_TTI], INT_V_TTI) },
     { FLDATA (DONE, int_hwre[API_TTI], INT_V_TTI) },
-#if defined (KSR28)
-    { ORDATA (TTI_STATE, tti_state, (TTI_WIDTH + 3)), REG_HRO },
+#if defined (PDP15)
+    { FLDATA (FDPX, tti_fdpx, 0) },
 #endif
     { DRDATA (POS, tti_unit.pos, T_ADDR_W), PV_LEFT },
-    { DRDATA (TIME, tti_unit.wait, 24), REG_NZ + PV_LEFT },
+    { DRDATA (TIME, tti_unit.wait, 24), PV_LEFT },
     { NULL }
     };
 
 MTAB tti_mod[] = {
 #if !defined (KSR28)
-    { UNIT_KSR+UNIT_8B, UNIT_KSR, "KSR", "KSR", &tty_set_mode },
-    { UNIT_KSR+UNIT_8B, 0       , "7b" , "7B" , &tty_set_mode },
-    { UNIT_KSR+UNIT_8B, UNIT_8B , "8b" , "8B" , &tty_set_mode },
-    { UNIT_HDX, 0       , "full duplex", "FDX", NULL },
-    { UNIT_HDX, UNIT_HDX, "half duplex", "HDX", NULL },
+    { TT_MODE, TT_MODE_KSR, "KSR", "KSR", &tty_set_mode },
+    { TT_MODE, TT_MODE_7B,  "7b",  "7B",  &tty_set_mode },
+    { TT_MODE, TT_MODE_8B,  "8b",  "8B",  &tty_set_mode },
+    { TT_MODE, TT_MODE_7P,  "7b",  NULL,  NULL },
 #endif
+    { TTUF_HDX, 0       , "full duplex", "FDX", NULL },
+    { TTUF_HDX, TTUF_HDX, "half duplex", "HDX", NULL },
     { MTAB_XTD|MTAB_VDV, 0, "DEVNO", NULL, NULL, &show_devno, NULL },
     { 0 }
     };
@@ -323,23 +352,12 @@ DEVICE tti_dev = {
    tto_dev      TTO device descriptor
    tto_unit     TTO unit
    tto_reg      TTO register list
-   tto_trans    Baudot to ASCII table
 */
 
 #if defined (KSR28)
 #define TTO_WIDTH       5
 #define TTO_FIGURES     (1 << TTO_WIDTH)
 
-static const char tto_trans[64] = {
-     0 ,'T',015,'O',' ','H','N','M',
-    012,'L','R','G','I','P','C','V',
-    'E','Z','D','B','S','Y','F','X',
-    'A','W','J', 0 ,'U','Q','K', 0,
-     0 ,'5','\r','9',' ','#',',','.',
-    012,')','4','&','8','0',':',';',
-    '3','"','$','?','\a','6','!','/',
-    '-','2','\'',0 ,'7','1','(', 0
-    };
 #else
 
 #define TTO_WIDTH       8
@@ -349,15 +367,15 @@ static const char tto_trans[64] = {
 
 DIB tto_dib = { DEV_TTO, 1, &tto_iors, { &tto } };
 
-UNIT tto_unit = { UDATA (&tto_svc, UNIT_KSR, 0), 1000 };
+UNIT tto_unit = { UDATA (&tto_svc, TT_MODE_KSR, 0), 1000 };
 
 REG tto_reg[] = {
     { ORDATA (BUF, tto_unit.buf, TTO_WIDTH) },
+#if defined (KSR28)
+    { FLDATA (SHIFT, tty_shift, 0), REG_HRO },
+#endif
     { FLDATA (INT, int_hwre[API_TTO], INT_V_TTO) },
     { FLDATA (DONE, int_hwre[API_TTO], INT_V_TTO) },
-#if defined (KSR28)
-    { FLDATA (TTO_STATE, tto_state, 0), REG_HRO },
-#endif
     { DRDATA (POS, tto_unit.pos, T_ADDR_W), PV_LEFT },
     { DRDATA (TIME, tto_unit.wait, 24), PV_LEFT },
     { NULL }
@@ -365,9 +383,10 @@ REG tto_reg[] = {
 
 MTAB tto_mod[] = {
 #if !defined (KSR28)
-    { UNIT_KSR+UNIT_8B, UNIT_KSR, "KSR", "KSR", &tty_set_mode },
-    { UNIT_KSR+UNIT_8B, 0       , "7b" , "7B" , &tty_set_mode },
-    { UNIT_KSR+UNIT_8B, UNIT_8B , "8b" , "8B" , &tty_set_mode },
+    { TT_MODE, TT_MODE_KSR, "KSR", "KSR", &tty_set_mode },
+    { TT_MODE, TT_MODE_7B,  "7b",  "7B",  &tty_set_mode },
+    { TT_MODE, TT_MODE_8B,  "8b",  "8B",  &tty_set_mode },
+    { TT_MODE, TT_MODE_7P,  "7p",  "7P",  &tty_set_mode },
 #endif
     { MTAB_XTD|MTAB_VDV, 0, "DEVNO", NULL, NULL, &show_devno },
     { 0 }
@@ -389,14 +408,9 @@ if (pulse & 001) {                                      /* CLSF */
     if (TST_INT (CLK)) dat = dat | IOT_SKP;
     }
 if (pulse & 004) {                                      /* CLON/CLOF */
-    if (pulse & 040) {                                  /* CLON */
-        CLR_INT (CLK);                                  /* clear flag */
-        clk_state = 1;                                  /* clock on */
-        if (!sim_is_active (&clk_unit))                 /* already on? */
-            sim_activate (&clk_unit,                    /* start, calibr */
-                sim_rtc_init (clk_unit.wait));
-        }
-    else clk_reset (&clk_dev);                          /* CLOF */
+    CLR_INT (CLK);                                      /* clear flag */
+    if (pulse & 040) clk_state = 1;                     /* CLON */
+    else clk_state = 0;                                 /* CLOF */
     }
 return dat;
 }
@@ -407,16 +421,49 @@ t_stat clk_svc (UNIT *uptr)
 {
 int32 t;
 
+t = sim_rtc_calb (clk_tps);                             /* calibrate clock */
+tmxr_poll = t;                                          /* set mux poll */
+sim_activate (&clk_unit, t);                            /* reactivate unit */
+#if defined (PDP15)
+clk_task_upd (FALSE);                                   /* update task timer */
+#endif
 if (clk_state) {                                        /* clock on? */
     M[7] = (M[7] + 1) & DMASK;                          /* incr counter */
     if (M[7] == 0) SET_INT (CLK);                       /* ovrflo? set flag */
-    t = sim_rtc_calb (clk_tps);                         /* calibrate clock */
-    sim_activate (&clk_unit, t);                        /* reactivate unit */
-    tmxr_poll = t;                                      /* set mux poll */
     }
 return SCPE_OK;
 }
 
+#if defined (PDP15)
+
+/* Task timer update (PDP-15 XVM only)
+
+   The task timer increments monotonically at 100Khz. Since this can't be
+   simulated accurately, updates are done by interpolation since the last
+   reading.  The timer is also updated at clock events to keep the cycle
+   counters from wrapping around more than once between updates. */
+
+int32 clk_task_upd (t_bool clr)
+{
+uint32 delta, val, iusec10;
+uint32 cur = sim_grtime ();
+uint32 old = clk_task_timer;
+double usec10;
+
+if (cur > clk_task_last) delta = cur - clk_task_last;
+else delta = clk_task_last - cur;
+usec10 = ((((double) delta) * 100000.0) /
+    (((double) tmxr_poll) * ((double) clk_tps)));
+iusec10 = (int32) usec10;
+val = (clk_task_timer + iusec10) & DMASK;
+if (clr) clk_task_timer = 0;
+else clk_task_timer = val;
+clk_task_last = cur;
+return ((int32) val);
+}
+
+#endif
+
 /* IORS service */
 
 int32 clk_iors (void)
@@ -428,11 +475,16 @@ return (TST_INT (CLK)? IOS_CLK: 0);
 
 t_stat clk_reset (DEVICE *dptr)
 {
+int32 t;
+
 CLR_INT (CLK);                                          /* clear flag */
-tmxr_poll = clk_unit.wait;                              /* set mux poll */
-if (!sim_is_running) {                                  /* RESET? */
+if (!sim_is_running) {                                  /* RESET (not CAF)? */
+    t = sim_rtc_init (clk_unit.wait);                   /* init calibration */
+    tmxr_poll = t;                                      /* set mux poll */
+    sim_activate_abs (&clk_unit, t);                    /* activate unit */
     clk_state = 0;                                      /* clock off */
-    sim_cancel (&clk_unit);                             /* stop clock */
+    clk_task_timer = 0;
+    clk_task_last = 0;
     }
 return SCPE_OK;
 }
@@ -455,6 +507,19 @@ fprintf (st, (clk_tps == 50)? "50Hz": "60Hz");
 return SCPE_OK;
 }
 
+/* Paper tape reader out-of-tape handling
+
+   The PDP-4 and PDP-7 readers behaved like most early DEC readers; when
+   they ran out of tape, they hung.  It was up to the program to sense this
+   condition by running a timer.
+
+   The PDP-9 reader controller synthesized the out of tape condition by
+   noticing whether there was a transition on the feed hole within a window.
+   The out-of-tape flag was treated like the reader flag in most cases.
+
+   The PDP-15 reader controller received the out-of-tape flag as a static
+   condition from the reader itself and simply reported it via IORS. */
+
 /* Paper tape reader: IOT routine */
 
 int32 ptr (int32 dev, int32 pulse, int32 dat)
@@ -463,12 +528,15 @@ if (pulse & 001) {                                      /* RSF */
     if (TST_INT (PTR)) dat = dat | IOT_SKP;
     }
 if (pulse & 002) {                                      /* RRB, RCF */
-    CLR_INT (PTR);                                      /* clear done */
+    CLR_INT (PTR);                                      /* clear flag */
     dat = dat | ptr_unit.buf;                           /* return buffer */
     }
 if (pulse & 004) {                                      /* RSA, RSB */
     ptr_state = (pulse & 040)? 18: 0;                   /* set mode */
-    CLR_INT (PTR);                                      /* clear done */
+    CLR_INT (PTR);                                      /* clear flag */
+#if !defined (PDP15)                                    /* except on PDP15 */
+    ptr_err = 0;                                        /* clear error */
+#endif
     ptr_unit.buf = 0;                                   /* clear buffer */
     sim_activate (&ptr_unit, ptr_unit.wait);
     }
@@ -483,15 +551,15 @@ int32 temp;
 
 if ((ptr_unit.flags & UNIT_ATT) == 0) {                 /* attached? */
 #if defined (IOS_PTRERR)
-    SET_INT (PTR);                                      /* if err, set int */
-    ptr_err = 1;
+    SET_INT (PTR);                                      /* if err, set flag */
+    ptr_err = 1;                                        /* set error */
 #endif
     return IORETURN (ptr_stopioe, SCPE_UNATT);
-	}
+    }
 if ((temp = getc (ptr_unit.fileref)) == EOF) {          /* end of file? */
 #if defined (IOS_PTRERR)
-    SET_INT (PTR);                                      /* if err, set done */
-    ptr_err = 1;
+    SET_INT (PTR);                                      /* if err, set flag */
+    ptr_err = 1;                                        /* set error */
 #endif
     if (feof (ptr_unit.fileref)) {
         if (ptr_stopioe) printf ("PTR end of file\n");
@@ -508,6 +576,8 @@ if (ptr_state == 0) {                                   /* ASCII */
             ptr_unit.buf = ptr_unit.buf ^ 0200;         /* count bits */
         ptr_unit.buf = ptr_unit.buf ^ 0200;             /* set even parity */
         }
+    else if (ptr_unit.flags & UNIT_KASCII)              /* KSR ASCII? */
+        ptr_unit.buf = (temp | 0200) & 0377;            /* forced parity */
     else ptr_unit.buf = temp & 0377;
     }
 else if (temp & 0200) {                                 /* binary */
@@ -527,7 +597,12 @@ t_stat ptr_reset (DEVICE *dptr)
 ptr_state = 0;                                          /* clear state */
 ptr_unit.buf = 0;
 CLR_INT (PTR);                                          /* clear flag */
-ptr_err = (ptr_unit.flags & UNIT_ATT)? 0: 1;
+#if defined (PDP15)                                     /* PDP15, static err */
+if (((ptr_unit.flags & UNIT_ATT) == 0) || feof (ptr_unit.fileref))
+    ptr_err = 1;
+else
+#endif
+ptr_err = 0;                                            /* all other, clr err */
 sim_cancel (&ptr_unit);                                 /* deactivate unit */
 return SCPE_OK;
 }
@@ -550,18 +625,23 @@ t_stat ptr_attach (UNIT *uptr, char *cptr)
 t_stat reason;
 
 reason = attach_unit (uptr, cptr);
-ptr_err = (ptr_unit.flags & UNIT_ATT)? 0: 1;
-ptr_unit.flags = ptr_unit.flags & ~UNIT_RASCII;
+if (reason != SCPE_OK) return reason;
+ptr_err = 0;                                             /* attach clrs error */
+ptr_unit.flags = ptr_unit.flags & ~(UNIT_RASCII|UNIT_KASCII);
 if (sim_switches & SWMASK ('A'))
     ptr_unit.flags = ptr_unit.flags | UNIT_RASCII;
-return reason;
+if (sim_switches & SWMASK ('K'))
+    ptr_unit.flags = ptr_unit.flags | UNIT_KASCII;
+return SCPE_OK;
 }
 
 /* Detach routine */
 
 t_stat ptr_detach (UNIT *uptr)
 {
+#if defined (PDP15)
 ptr_err = 1;
+#endif
 ptr_unit.flags = ptr_unit.flags & ~UNIT_RASCII;
 return detach_unit (uptr);
 }
@@ -599,7 +679,7 @@ for (;;) {                                              /* word loop */
             }
         else if (val == OP_HLT) return STOP_HALT;
         break;
-		}
+        }
     else if (MEM_ADDR_OK (origin)) M[origin++] = val;
     }
 return SCPE_FMT;
@@ -803,7 +883,7 @@ return dat;
 
 t_stat ptp_svc (UNIT *uptr)
 {
-SET_INT (PTP);                                          /* set done flag */
+SET_INT (PTP);                                          /* set flag */
 if ((ptp_unit.flags & UNIT_ATT) == 0) {                 /* not attached? */
     ptp_err = 1;                                        /* set error */
     return IORETURN (ptp_stopioe, SCPE_UNATT);
@@ -852,7 +932,8 @@ t_stat ptp_attach (UNIT *uptr, char *cptr)
 t_stat reason;
 
 reason = attach_unit (uptr, cptr);
-ptp_err = (ptp_unit.flags & UNIT_ATT)? 0: 1;
+if (reason != SCPE_OK) return reason;
+ptp_err = 0;
 ptp_unit.flags = ptp_unit.flags & ~UNIT_PASCII;
 if (sim_switches & SWMASK ('A'))
     ptp_unit.flags = ptp_unit.flags | UNIT_PASCII;
@@ -875,9 +956,13 @@ int32 tti (int32 dev, int32 pulse, int32 dat)
 if (pulse & 001) {                                      /* KSF */
     if (TST_INT (TTI)) dat = dat | IOT_SKP;
     }
-if (pulse & 002) {                                      /* KRB */
+if (pulse & 002) {                                      /* KRS/KRB */
     CLR_INT (TTI);                                      /* clear flag */
     dat = dat | tti_unit.buf & TTI_MASK;                /* return buffer */
+#if defined (PDP15)
+    if (pulse & 020) tti_fdpx = 1;                      /* KRS? */
+    else tti_fdpx = 0;                                  /* no, KRB */
+#endif
     }
 if (pulse & 004) {                                      /* IORS */
     dat = dat | upd_iors ();
@@ -890,41 +975,48 @@ return dat;
 t_stat tti_svc (UNIT *uptr)
 {
 #if defined (KSR28)                                     /* Baudot... */
-int32 c;
+int32 in, c, out;
 
-sim_activate (uptr, uptr->wait);                        /* continue poll */
-if (tti_state & TTI_2ND) {                              /* char waiting? */
-    uptr->buf = tti_state & TTI_MASK;                   /* return char */
-    tti_state = tti_state & ~TTI_2ND;                   /* not waiting */
+sim_activate (uptr, KBD_WAIT (uptr->wait, tmxr_poll));  /* continue poll */
+if (tti_2nd) {                                          /* char waiting? */
+    uptr->buf = tti_2nd;                                /* return char */
+    tti_2nd = 0;                                        /* not waiting */
     }
 else {
-    if ((c = sim_poll_kbd ()) < SCPE_KFLAG) return c;
-    c = tti_trans[c & 0177];                            /* translate char */
+    if ((in = sim_poll_kbd ()) < SCPE_KFLAG) return in;
+    c = asc_to_baud[in & 0177];                         /* translate char */
     if (c == 0) return SCPE_OK;                         /* untranslatable? */
-    if (((c & TTI_FIGURES) == (tti_state & TTI_FIGURES)) ||
-        (c & TTI_BOTH)) uptr->buf = c & TTI_MASK;
-    else {
-        uptr->buf = (c & TTI_FIGURES)?
-            BAUDOT_FIGURES: BAUDOT_LETTERS;
-        tti_state = c | TTI_2ND;                        /* set 2nd waiting */
+    if ((c & TTI_BOTH) ||                               /* case insensitive? */
+        (((c & TTI_FIGURES)? 1: 0) == tty_shift))       /* right case? */
+        uptr->buf = c & TTI_MASK;
+    else {                                              /* send case change */
+        if (c & TTI_FIGURES) {                          /* to figures? */
+            uptr->buf = BAUDOT_FIGURES;
+            tty_shift = 1;
+            }
+        else {                                          /* no, to letters */
+            uptr->buf = BAUDOT_LETTERS;
+            tty_shift = 0;
+            }
+        tti_2nd = c & TTI_MASK;                         /* save actual char */
+        }
+    if ((uptr->flags & TTUF_HDX) &&                     /* half duplex? */
+        ((out = sim_tt_outcvt (in, TT_GET_MODE (uptr->flags) | TTUF_KSR)) >= 0)) {
+        sim_putchar (out);
+        tto_unit.pos = tto_unit.pos + 1;
         }
     }
 
 #else                                                   /* ASCII... */
 int32 c, out;
 
-sim_activate (uptr, uptr->wait);                        /* continue poll */
+sim_activate (uptr, KBD_WAIT (uptr->wait, tmxr_poll));  /* continue poll */
 if ((c = sim_poll_kbd ()) < SCPE_KFLAG) return c;       /* no char or error? */
 out = c & 0177;                                         /* mask echo to 7b */
 if (c & SCPE_BREAK) c = 0;                              /* break? */
-else if (uptr->flags & UNIT_KSR) {                      /* KSR? */
-    if (islower (out)) out = toupper (out);             /* convert to UC */
-    c = out | 0200;                                     /* set TTY bit */
-	}
-else c = c & ((uptr->flags & UNIT_8B)? 0377: 0177);     /* no, 7b/8b */
-if ((uptr->flags & UNIT_HDX) && out &&                  /* half duplex and */
-    (!(tto_unit.flags & UNIT_KSR) ||                    /* 7b/8b or */
-    ((out >= 007) && (out <= 0137)))) {                 /* in range? */
+else c = sim_tt_inpcvt (c, TT_GET_MODE (uptr->flags) | TTUF_KSR);
+if ((uptr->flags & TTUF_HDX) && !tti_fdpx && out &&     /* half duplex and */
+    ((out = sim_tt_outcvt (out, TT_GET_MODE (uptr->flags) | TTUF_KSR)) >= 0)) {
     sim_putchar (out);                                  /* echo */
     tto_unit.pos = tto_unit.pos + 1;
     }
@@ -948,9 +1040,11 @@ return (TST_INT (TTI)? IOS_TTI: 0);
 t_stat tti_reset (DEVICE *dptr)
 {
 tti_unit.buf = 0;                                       /* clear buffer */
-tti_state = 0;                                          /* clear state */
+tti_2nd = 0;
+tty_shift = 0;                                          /* clear state */
+tti_fdpx = 0;                                           /* clear dpx mode */
 CLR_INT (TTI);                                          /* clear flag */
-sim_activate (&tti_unit, tti_unit.wait);                /* activate unit */
+sim_activate_abs (&tti_unit, KBD_WAIT (tti_unit.wait, tmxr_poll));
 return SCPE_OK;
 }
 
@@ -978,19 +1072,16 @@ t_stat r;
 
 #if defined (KSR28)                                     /* Baudot... */
 if (uptr->buf == BAUDOT_FIGURES)                        /* set figures? */
-    tto_state = TTO_FIGURES;
+    tty_shift = 1;
 else if (uptr->buf == BAUDOT_LETTERS)                   /* set letters? */
-    tto_state = 0;
+    tty_shift = 0;
 else {
-    c = tto_trans[uptr->buf + tto_state];               /* translate */
+    c = baud_to_asc[uptr->buf | (tty_shift << 5)];      /* translate */
 
 #else
-c = uptr->buf & 0177;                                   /* assume 7b or KSR */
-if (!(uptr->flags & UNIT_KSR) ||                        /* 7b/8b or */
-      ((c >= 007) && (c <= 0137))) {                    /* in range? */
-    if ((uptr->flags & UNIT_KSR) && islower (c))        /* KSR? */
-        c = toupper (c);
-    else if (tto_unit.flags & UNIT_8B) c = uptr->buf;
+c = sim_tt_outcvt (uptr->buf, TT_GET_MODE (uptr->flags) | TTUF_KSR);
+if (c >= 0) {
+
 #endif
 
     if ((r = sim_putchar_s (c)) != SCPE_OK) {           /* output; error? */
@@ -1015,7 +1106,7 @@ return (TST_INT (TTO)? IOS_TTO: 0);
 t_stat tto_reset (DEVICE *dptr)
 {
 tto_unit.buf = 0;                                       /* clear buffer */
-tto_state = 0;                                          /* clear state */
+tty_shift = 0;                                          /* clear state */
 CLR_INT (TTO);                                          /* clear flag */
 sim_cancel (&tto_unit);                                 /* deactivate unit */
 return SCPE_OK;
@@ -1025,7 +1116,7 @@ return SCPE_OK;
 
 t_stat tty_set_mode (UNIT *uptr, int32 val, char *cptr, void *desc)
 {
-tti_unit.flags = (tti_unit.flags & ~(UNIT_KSR | UNIT_8B)) | val;
-tto_unit.flags = (tto_unit.flags & ~(UNIT_KSR | UNIT_8B)) | val;
+tti_unit.flags = (tti_unit.flags & ~TT_MODE) | val;
+tto_unit.flags = (tto_unit.flags & ~TT_MODE) | val;
 return SCPE_OK;
 }
