@@ -23,6 +23,7 @@
    used in advertising or otherwise to promote the sale, use or other dealings
    in this Software without prior written authorization from Robert M Supnik.
 
+   03-Nov-05    RMS     Added 780 stop codes
    04-Sep-05    RMS     Fixed missing assignment (found by Peter Schorn)
    16-Aug-05    RMS     Fixed C++ declaration and cast problems
    15-Sep-04    RMS     Fixed bugs in character display and parse
@@ -97,6 +98,8 @@ const char *sim_stop_messages[] = {
     "Fatal RQDX3 error",
     "Infinite loop",
     "Sanity timer expired",
+    "Software done",
+    "Reboot requested",
     "Unknown error",
     "Unknown abort code"
     };
@@ -673,7 +676,7 @@ const char *opcode[] = {
 "CVTPS", "CVTSP", "INDEX", "CRC", "PROBER", "PROBEW", "INSQUE", "REMQUE",
 "BSBB", "BRB", "BNEQ", "BEQL", "BGTR", "BLEQ", "JSB", "JMP",
 "BGEQ", "BLSS", "BGTRU", "BLEQU", "BVC", "BVS", "BGEQU", "BLSSU",
-"ADDP4", "ADDP6", "SUBP4", "SUBP6", "CVTPT", "MULP6", "CVTTP", "DIVP",
+"ADDP4", "ADDP6", "SUBP4", "SUBP6", "CVTPT", "MULP", "CVTTP", "DIVP",
 "MOVC3", "CMPC3", "SCANC", "SPANC", "MOVC5", "CMPC5", "MOVTC", "MOVTUC",
 "BSBW", "BRW", "CVTWL", "CVTWB", "MOVP", "CMPP3", "CVTPL", "CMPP4",
 "EDITPC", "MATCHC", "LOCC", "SKPC", "MOVZWL", "ACBW", "MOVAW", "PUSHAW",
