@@ -1,6 +1,6 @@
 /* sim_sock.h: OS-dependent socket routines header file
 
-   Copyright (c) 2001-2004, Robert M Supnik
+   Copyright (c) 2001-2005, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -19,56 +19,58 @@
    IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
    CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-   Except as contained in this notice, the name of Robert M Supnik shall not
-   be used in advertising or otherwise to promote the sale, use or other dealings
+   Except as contained in this notice, the name of Robert M Supnik shall not be
+   used in advertising or otherwise to promote the sale, use or other dealings
    in this Software without prior written authorization from Robert M Supnik.
 
-   20-Aug-04	HV	Added missing definition for OS/2 (from Holger Veit)
+   14-Apr-05    RMS     Added WSAEINPROGRESS (from Tim Riker)
+   20-Aug-04    HV      Added missing definition for OS/2 (from Holger Veit)
    22-Oct-03    MP      Changed WIN32 winsock include to use winsock2.h to
                         avoid a conflict if sim_sock.h and sim_ether.h get
                         included by the same module.
-   20-Mar-03	RMS	Added missing timerclear definition for VMS (from
-			Robert Alan Byer)
-   15-Feb-03	RMS	Added time.h for EMX (from Holger Veit)
-   17-Dec-02	RMS	Added sim_connect_sock
-   08-Oct-02	RMS	Revised for .NET compatibility
-   20-Aug-02	RMS	Changed calling sequence for sim_accept_conn
-   30-Apr-02	RMS	Changed VMS stropts include to ioctl
-   06-Feb-02	RMS	Added VMS support from Robert Alan Byer
-   16-Sep-01	RMS	Added Macintosh support from Peter Schorn
+   20-Mar-03    RMS     Added missing timerclear definition for VMS (from
+                        Robert Alan Byer)
+   15-Feb-03    RMS     Added time.h for EMX (from Holger Veit)
+   17-Dec-02    RMS     Added sim_connect_sock
+   08-Oct-02    RMS     Revised for .NET compatibility
+   20-Aug-02    RMS     Changed calling sequence for sim_accept_conn
+   30-Apr-02    RMS     Changed VMS stropts include to ioctl
+   06-Feb-02    RMS     Added VMS support from Robert Alan Byer
+   16-Sep-01    RMS     Added Macintosh support from Peter Schorn
 */
 
 #ifndef _SIM_SOCK_H_
-#define _SIM_SOCK_H_	0
+#define _SIM_SOCK_H_    0
 
-#if defined (_WIN32)					/* Windows */
-#undef INT_PTR						/* hack, hack */
+#if defined (_WIN32)                                    /* Windows */
+#undef INT_PTR                                          /* hack, hack */
 #include <winsock2.h>
 
-#elif !defined (__OS2__) || defined (__EMX__)		/* VMS, Mac, Unix, OS/2 EMX */
-#define WSAGetLastError()	errno			/* Windows macros */
-#define SOCKET		int32
-#define WSAEWOULDBLOCK	EWOULDBLOCK
-#define INVALID_SOCKET	-1 
-#define SOCKET_ERROR	-1
-#include <sys/types.h>					/* for fcntl, getpid */
-#include <sys/socket.h>					/* for sockets */
+#elif !defined (__OS2__) || defined (__EMX__)           /* VMS, Mac, Unix, OS/2 EMX */
+#define WSAGetLastError()       errno                   /* Windows macros */
+#define SOCKET          int32
+#define WSAEWOULDBLOCK  EWOULDBLOCK
+#define WSAEINPROGRESS  EINPROGRESS
+#define INVALID_SOCKET  -1 
+#define SOCKET_ERROR    -1
+#include <sys/types.h>                                  /* for fcntl, getpid */
+#include <sys/socket.h>                                 /* for sockets */
 #include <fcntl.h>
 #include <unistd.h>
-#include <netinet/in.h>					/* for sockaddr_in */
+#include <netinet/in.h>                                 /* for sockaddr_in */
 #include <netdb.h>
-#include <sys/time.h>					/* for EMX */
+#include <sys/time.h>                                   /* for EMX */
 #endif
 
-#if defined (VMS)					/* VMS unique */
-#include <ioctl.h>					/* for ioctl */
+#if defined (VMS)                                       /* VMS unique */
+#include <ioctl.h>                                      /* for ioctl */
 #if !defined (timerclear)
 #define timerclear(tvp)         (tvp)->tv_sec = (tvp)->tv_usec = 0
 #endif
 #endif
-#if defined(__EMX__)					/* OS/2 unique */
+#if defined(__EMX__)                                    /* OS/2 unique */
 #if !defined (timerclear)
-#define timerclear(tvp)		(tvp)->tv_sec = (tvp)->tv_usec = 0
+#define timerclear(tvp)         (tvp)->tv_sec = (tvp)->tv_usec = 0
 #endif
 #endif
 
